@@ -6,7 +6,7 @@
 #
 # See the help page for usage information.
 #
-# © Copyright EnterpriseDB UK Limited 2019-2021
+# © Copyright EnterpriseDB UK Limited 2019-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -127,9 +127,12 @@ def connectivity_test(config):
     """
     ssh_command = build_ssh_command(config)
     try:
-        output = subprocess.Popen(ssh_command, stdout=subprocess.PIPE).communicate()
+        pipe = subprocess.Popen(
+            ssh_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
+        )
+        output = pipe.communicate()
         print(output[0].decode("utf-8"))
-        sys.exit(0)
+        sys.exit(pipe.returncode)
     except subprocess.CalledProcessError as e:
         exit_with_error("Impossible to invoke remote put-wal: %s" % e)
 
