@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -16,14 +16,18 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>
 
-import bz2
-import gzip
 import logging
 import os
-import shutil
-from io import BytesIO, RawIOBase
+import requests
+from io import BytesIO, RawIOBase, SEEK_END
 
-from barman.cloud import CloudInterface, CloudProviderError
+from barman.clients.cloud_compression import decompress_to_file
+from barman.cloud import (
+    CloudInterface,
+    CloudProviderError,
+    DecompressingStreamingIO,
+    DEFAULT_DELIMITER,
+)
 
 try:
     # Python 3.x
@@ -102,7 +106,37 @@ class AzureCloudInterface(CloudInterface):
     # MAX_ARCHIVE_SIZE - so we set a maximum of 1TB per file
     MAX_ARCHIVE_SIZE = 1 << 40
 
-    def __init__(self, url, jobs=2, encryption_scope=None):
+    # The size of each chunk in a single object upload when the size of the
+    # object exceeds max_single_put_size. We default to 2MB in order to
+    # allow the default max_concurrency of 8 to be achieved when uploading
+    # uncompressed WAL segments of the default 16MB size.
+    DEFAULT_MAX_BLOCK_SIZE = 2 << 20
+
+    # The maximum amount of concurrent chunks allowed in a single object upload
+    # where the size exceeds max_single_put_size. We default to 8 based on
+    # experiments with in-region and inter-region transfers within Azure.
+    DEFAULT_MAX_CONCURRENCY = 8
+
+    # The largest file size which will be uploaded in a single PUT request. This
+    # should be lower than the size of the compressed WAL segment in order to
+    # force the Azure client to use concurrent chunk upload for archiving WAL files.
+    DEFAULT_MAX_SINGLE_PUT_SIZE = 4 << 20
+
+    # The maximum size of the requests connection pool used by the Azure client
+    # to upload objects.
+    REQUESTS_POOL_MAXSIZE = 32
+
+    def __init__(
+        self,
+        url,
+        jobs=2,
+        encryption_scope=None,
+        credential=None,
+        tags=None,
+        max_block_size=DEFAULT_MAX_BLOCK_SIZE,
+        max_concurrency=DEFAULT_MAX_CONCURRENCY,
+        max_single_put_size=DEFAULT_MAX_SINGLE_PUT_SIZE,
+    ):
         """
         Create a new Azure Blob Storage interface given the supplied acccount url
 
@@ -113,8 +147,13 @@ class AzureCloudInterface(CloudInterface):
         super(AzureCloudInterface, self).__init__(
             url=url,
             jobs=jobs,
+            tags=tags,
         )
         self.encryption_scope = encryption_scope
+        self.credential = credential
+        self.max_block_size = max_block_size
+        self.max_concurrency = max_concurrency
+        self.max_single_put_size = max_single_put_size
 
         parsed_url = urlparse(url)
         if parsed_url.netloc.endswith(AZURE_BLOB_STORAGE_DOMAIN):
@@ -152,12 +191,16 @@ class AzureCloudInterface(CloudInterface):
         """
         Create a new session
         """
-        if "AZURE_STORAGE_CONNECTION_STRING" in os.environ:
+        if self.credential:
+            # Any supplied credential takes precedence over the environment
+            credential = self.credential
+        elif "AZURE_STORAGE_CONNECTION_STRING" in os.environ:
             logging.info("Authenticating to Azure with connection string")
             self.container_client = ContainerClient.from_connection_string(
                 conn_str=os.getenv("AZURE_STORAGE_CONNECTION_STRING"),
                 container_name=self.bucket_name,
             )
+            return
         else:
             if "AZURE_STORAGE_SAS_TOKEN" in os.environ:
                 logging.info("Authenticating to Azure with SAS token")
@@ -174,11 +217,17 @@ class AzureCloudInterface(CloudInterface):
                 except ImportError:
                     raise SystemExit("Missing required python module: azure-identity")
                 credential = DefaultAzureCredential()
-            self.container_client = ContainerClient(
-                account_url=self.account_url,
-                container_name=self.bucket_name,
-                credential=credential,
-            )
+        session = requests.Session()
+        adapter = requests.adapters.HTTPAdapter(pool_maxsize=self.REQUESTS_POOL_MAXSIZE)
+        session.mount("https://", adapter)
+        self.container_client = ContainerClient(
+            account_url=self.account_url,
+            container_name=self.bucket_name,
+            credential=credential,
+            max_single_put_size=self.max_single_put_size,
+            max_block_size=self.max_block_size,
+            session=session,
+        )
 
     @property
     def _extra_upload_args(self):
@@ -249,7 +298,7 @@ class AzureCloudInterface(CloudInterface):
                 for v in self._walk_blob_tree(child):
                     yield v
 
-    def list_bucket(self, prefix="", delimiter="/"):
+    def list_bucket(self, prefix="", delimiter=DEFAULT_DELIMITER):
         """
         List bucket content in a directory manner
 
@@ -273,40 +322,63 @@ class AzureCloudInterface(CloudInterface):
         """
         obj = self.container_client.download_blob(key)
         with open(dest_path, "wb") as dest_file:
-            if not decompress:
+            if decompress is None:
                 obj.download_to_stream(dest_file)
                 return
             blob = StreamingBlobIO(obj)
-            if decompress == "gzip":
-                source_file = gzip.GzipFile(fileobj=blob, mode="rb")
-            elif decompress == "bzip2":
-                source_file = bz2.BZ2File(blob, "rb")
-            with source_file:
-                shutil.copyfileobj(source_file, dest_file)
+            decompress_to_file(blob, dest_file, decompress)
 
-    def remote_open(self, key):
+    def remote_open(self, key, decompressor=None):
         """
         Open a remote Azure Blob Storage object and return a readable stream
 
         :param str key: The key identifying the object to open
+        :param barman.clients.cloud_compression.ChunkedCompressor decompressor:
+          A ChunkedCompressor object which will be used to decompress chunks of bytes
+          as they are read from the stream
         :return: A file-like object from which the stream can be read or None if
           the key does not exist
         """
         try:
             obj = self.container_client.download_blob(key)
-            return StreamingBlobIO(obj)
+            resp = StreamingBlobIO(obj)
+            if decompressor:
+                return DecompressingStreamingIO(resp, decompressor)
+            else:
+                return resp
         except ResourceNotFoundError:
             return None
 
-    def upload_fileobj(self, fileobj, key):
+    def upload_fileobj(
+        self,
+        fileobj,
+        key,
+        override_tags=None,
+    ):
         """
         Synchronously upload the content of a file-like object to a cloud key
 
         :param fileobj IOBase: File-like object to upload
         :param str key: The key to identify the uploaded object
+        :param List[tuple] override_tags: List of tags as k,v tuples to be added to the
+          uploaded object
         """
+        # Find length of the file so we can pass it to the Azure client
+        fileobj.seek(0, SEEK_END)
+        length = fileobj.tell()
+        fileobj.seek(0)
+
+        extra_args = self._extra_upload_args.copy()
+        tags = override_tags or self.tags
+        if tags is not None:
+            extra_args["tags"] = dict(tags)
         self.container_client.upload_blob(
-            name=key, data=fileobj, overwrite=True, **self._extra_upload_args
+            name=key,
+            data=fileobj,
+            overwrite=True,
+            length=length,
+            max_concurrency=self.max_concurrency,
+            **extra_args
         )
 
     def create_multipart_upload(self, key):
@@ -353,7 +425,10 @@ class AzureCloudInterface(CloudInterface):
         """
         blob_client = self.container_client.get_blob_client(key)
         block_list = [part["PartNumber"] for part in parts]
-        blob_client.commit_block_list(block_list, **self._extra_upload_args)
+        extra_args = self._extra_upload_args.copy()
+        if self.tags is not None:
+            extra_args["tags"] = dict(self.tags)
+        blob_client.commit_block_list(block_list, **extra_args)
 
     def _abort_multipart_upload(self, upload_metadata, key):
         """
