@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -457,9 +457,22 @@ class RecoveryExecutor(object):
         """
         target_epoch = None
         target_datetime = None
+
+        # Calculate the integer value of TLI if a keyword is provided
+        calculated_target_tli = target_tli
+        if target_tli and type(target_tli) is str:
+            if target_tli == "current":
+                calculated_target_tli = backup_info.timeline
+            elif target_tli == "latest":
+                valid_timelines = self.backup_manager.get_latest_archived_wals_info()
+                calculated_target_tli = int(max(valid_timelines.keys()))
+            elif not target_tli.isdigit():
+                raise ValueError("%s is not a valid timeline keyword" % target_tli)
+
         d_immediate = backup_info.version >= 90400 and target_immediate
         d_lsn = backup_info.version >= 100000 and target_lsn
-        d_tli = target_tli and target_tli != backup_info.timeline
+        d_tli = calculated_target_tli != backup_info.timeline and calculated_target_tli
+
         # Detect PITR
         if target_time or target_xid or d_tli or target_name or d_immediate or d_lsn:
             recovery_info["is_pitr"] = True
@@ -502,7 +515,7 @@ class RecoveryExecutor(object):
                 targets["xid"] = str(target_xid)
             if d_lsn:
                 targets["lsn"] = str(d_lsn)
-            if d_tli and target_tli != backup_info.timeline:
+            if d_tli:
                 targets["timeline"] = str(d_tli)
             if target_name:
                 targets["name"] = str(target_name)
