@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -50,13 +50,17 @@ from barman.utils import (
     BarmanEncoder,
     check_non_negative,
     check_positive,
+    check_tli,
     configure_logging,
     drop_privileges,
     force_str,
     get_log_levels,
     parse_log_level,
+    SHA256,
 )
 from barman.xlog import check_archive_usable
+from barman.backup_manifest import BackupManifest
+from barman.storage.local_file_manager import LocalFileManager
 
 _logger = logging.getLogger(__name__)
 
@@ -580,7 +584,7 @@ def rebuild_xlogdb(args):
             completer=server_completer,
             help="specifies the server name for the command ",
         ),
-        argument("--target-tli", help="target timeline", type=check_positive),
+        argument("--target-tli", help="target timeline", type=check_tli),
         argument(
             "--target-time",
             help="target time. You can use any valid unambiguous representation. "
@@ -772,7 +776,7 @@ def recover(args):
     if hasattr(args, "get_wal"):
         if args.get_wal:
             server.config.recovery_options.add(RecoveryOptions.GET_WAL)
-        else:
+        elif RecoveryOptions.GET_WAL in server.config.recovery_options:
             server.config.recovery_options.remove(RecoveryOptions.GET_WAL)
     if args.jobs is not None:
         server.config.parallel_jobs = args.jobs
@@ -869,11 +873,13 @@ def show_servers(args):
 
         # If the server has been manually disabled
         if not server.config.active:
-            name += " (inactive)"
+            description = "(inactive)"
         # If server has configuration errors
         elif server.config.disabled:
-            name += " (WARNING: disabled)"
-        output.init("show_server", name)
+            description = "(WARNING: disabled)"
+        else:
+            description = None
+        output.init("show_server", name, description=description)
         with closing(server):
             server.show()
     output.close_and_exit()
@@ -1419,6 +1425,65 @@ def check_backup(args):
     output.close_and_exit()
 
 
+@command(
+    [
+        argument(
+            "server_name",
+            completer=server_completer,
+            help="specifies the server name for the command ",
+        ),
+        argument(
+            "backup_id", completer=backup_completer, help="specifies the backup ID"
+        ),
+    ],
+    cmd_aliases=["verify"],
+)
+def verify_backup(args):
+    """
+    verify a backup for the given server and backup id
+    """
+    # get barman.server.Server
+    server = get_server(args)
+    # Raises an error if wrong backup
+    backup_info = parse_backup_id(server, args)
+    # get backup path
+    output.info("Verifying backup %s on server %s" % (args.backup_id, args.server_name))
+
+    server.backup_manager.verify_backup(backup_info)
+    output.close_and_exit()
+
+
+@command(
+    [
+        argument(
+            "server_name",
+            completer=server_completer,
+            help="specifies the server name for the command ",
+        ),
+        argument(
+            "backup_id", completer=backup_completer, help="specifies the backup ID"
+        ),
+    ],
+)
+def generate_manifest(args):
+    """
+    Generate a manifest-backup for the given server and backup id
+    """
+    server = get_server(args)
+    # Raises an error if wrong backup
+    backup_info = parse_backup_id(server, args)
+    # know context (remote backup? local?)
+
+    local_file_manager = LocalFileManager()
+    backup_manifest = BackupManifest(
+        backup_info.get_data_directory(), local_file_manager, SHA256()
+    )
+    backup_manifest.create_backup_manifest()
+
+    output.info("Backup %s is valid on server %s" % (args.backup_id, args.server_name))
+    output.close_and_exit()
+
+
 @command(
     [
         argument(
