@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -16,7 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-import argparse
 import logging
 import os
 import re
@@ -24,7 +23,14 @@ import tempfile
 from contextlib import closing
 from shutil import rmtree
 
-import barman
+from barman.clients.cloud_cli import (
+    add_tag_argument,
+    create_argument_parser,
+    GeneralErrorExit,
+    NetworkErrorExit,
+    OperationErrorExit,
+    UrlArgumentType,
+)
 from barman.cloud import (
     CloudBackupUploaderBarman,
     CloudBackupUploaderPostgres,
@@ -113,7 +119,7 @@ def main(args=None):
         cloud_interface = get_cloud_interface(config)
 
         if not cloud_interface.test_connectivity():
-            raise SystemExit(1)
+            raise NetworkErrorExit()
         # If test is requested, just exit after connectivity test
         elif config.test:
             raise SystemExit(0)
@@ -162,7 +168,7 @@ def main(args=None):
                 except PostgresConnectionError as exc:
                     logging.error("Cannot connect to postgres: %s", force_str(exc))
                     logging.debug("Exception details:", exc_info=exc)
-                    raise SystemExit(1)
+                    raise OperationErrorExit()
 
                 with closing(postgres):
                     uploader = CloudBackupUploaderPostgres(
@@ -173,7 +179,7 @@ def main(args=None):
     except KeyboardInterrupt as exc:
         logging.error("Barman cloud backup was interrupted by the user")
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise OperationErrorExit()
     except UnrecoverableHookScriptError as exc:
         logging.error("Barman cloud backup exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
@@ -181,7 +187,7 @@ def main(args=None):
     except Exception as exc:
         logging.error("Barman cloud backup exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise GeneralErrorExit()
     finally:
         # Remove the temporary directory and all the contained files
         rmtree(tempdir, ignore_errors=True)
@@ -194,39 +200,12 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser = argparse.ArgumentParser(
+    parser, s3_arguments, azure_arguments = create_argument_parser(
         description="This script can be used to perform a backup "
         "of a local PostgreSQL instance and ship "
         "the resulting tarball(s) to the Cloud. "
-        "Currently AWS S3 and Azure Blob Storage are supported.",
-        add_help=False,
-    )
-    parser.add_argument(
-        "destination_url",
-        help="URL of the cloud destination, such as a bucket in AWS S3."
-        " For example: `s3://bucket/path/to/folder`.",
-    )
-    parser.add_argument(
-        "server_name", help="the name of the server as configured in Barman."
-    )
-    parser.add_argument(
-        "-V", "--version", action="version", version="%%(prog)s %s" % barman.__version__
-    )
-    parser.add_argument("--help", action="help", help="show this help message and exit")
-    verbosity = parser.add_mutually_exclusive_group()
-    verbosity.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase output verbosity (e.g., -vv is more than -v)",
-    )
-    verbosity.add_argument(
-        "-q",
-        "--quiet",
-        action="count",
-        default=0,
-        help="decrease output verbosity (e.g., -qq is less than -q)",
+        "Currently AWS S3, Azure Blob Storage and Google Cloud Storage are supported.",
+        source_or_destination=UrlArgumentType.destination,
     )
     compression = parser.add_mutually_exclusive_group()
     compression.add_argument(
@@ -245,12 +224,12 @@ def parse_arguments(args=None):
         const="bz2",
         dest="compression",
     )
-    parser.add_argument(
-        "-t",
-        "--test",
-        help="Test cloud connectivity and exit",
-        action="store_true",
-        default=False,
+    compression.add_argument(
+        "--snappy",
+        help="snappy-compress the WAL while uploading to the cloud ",
+        action="store_const",
+        const="snappy",
+        dest="compression",
     )
     parser.add_argument(
         "-h",
@@ -289,22 +268,15 @@ def parse_arguments(args=None):
         default="100GB",
     )
     parser.add_argument(
-        "--cloud-provider",
-        help="The cloud provider to use as a storage backend",
-        choices=["aws-s3", "azure-blob-storage"],
-        default="aws-s3",
+        "-d",
+        "--dbname",
+        help="Database name or conninfo string for Postgres connection (default: postgres)",
+        default="postgres",
     )
-    s3_arguments = parser.add_argument_group(
-        "Extra options for the aws-s3 cloud provider"
-    )
-    s3_arguments.add_argument(
-        "--endpoint-url",
-        help="Override default S3 endpoint URL with the given one",
-    )
-    s3_arguments.add_argument(
-        "-P",
-        "--profile",
-        help="profile name (e.g. INI section in AWS credentials file)",
+    add_tag_argument(
+        parser,
+        name="tags",
+        help="Tags to be added to all uploaded files in cloud storage",
     )
     s3_arguments.add_argument(
         "-e",
@@ -313,15 +285,6 @@ def parse_arguments(args=None):
         "Allowed values: 'AES256'|'aws:kms'.",
         choices=["AES256", "aws:kms"],
     )
-    parser.add_argument(
-        "-d",
-        "--dbname",
-        help="Database name or conninfo string for Postgres connection (default: postgres)",
-        default="postgres",
-    )
-    azure_arguments = parser.add_argument_group(
-        "Extra options for the azure-blob-storage cloud provider"
-    )
     azure_arguments.add_argument(
         "--encryption-scope",
         help="The name of an encryption scope defined in the Azure Blob Storage "
