@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -15,12 +15,17 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
-import argparse
 import logging
 import os
 from contextlib import closing
 
-import barman
+from barman.clients.cloud_cli import (
+    CLIErrorExit,
+    create_argument_parser,
+    GeneralErrorExit,
+    NetworkErrorExit,
+    OperationErrorExit,
+)
 from barman.cloud import CloudBackupCatalog, configure_logging
 from barman.cloud_providers import get_cloud_interface
 from barman.utils import force_str
@@ -41,7 +46,7 @@ def main(args=None):
         logging.error(
             "Destination %s already exists and it is not empty", config.recovery_dir
         )
-        raise SystemExit(1)
+        raise OperationErrorExit()
 
     try:
         cloud_interface = get_cloud_interface(config)
@@ -52,14 +57,14 @@ def main(args=None):
             )
 
             if not cloud_interface.test_connectivity():
-                raise SystemExit(1)
+                raise NetworkErrorExit()
             # If test is requested, just exit after connectivity test
             elif config.test:
                 raise SystemExit(0)
 
             if not cloud_interface.bucket_exists:
                 logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
-                raise SystemExit(1)
+                raise OperationErrorExit()
 
             downloader.download_backup(
                 config.backup_id,
@@ -70,11 +75,11 @@ def main(args=None):
     except KeyboardInterrupt as exc:
         logging.error("Barman cloud restore was interrupted by the user")
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise OperationErrorExit()
     except Exception as exc:
         logging.error("Barman cloud restore exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise GeneralErrorExit()
 
 
 def parse_arguments(args=None):
@@ -84,49 +89,13 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser = argparse.ArgumentParser(
+    parser, _, _ = create_argument_parser(
         description="This script can be used to download a backup "
         "previously made with barman-cloud-backup command."
-        "Currently AWS S3 and Azure Blob Storage are supported.",
-        add_help=False,
-    )
-
-    parser.add_argument(
-        "source_url",
-        help="URL of the cloud source, such as a bucket in AWS S3."
-        " For example: `s3://bucket/path/to/folder`.",
-    )
-    parser.add_argument(
-        "server_name", help="the name of the server as configured in Barman."
+        "Currently AWS S3, Azure Blob Storage and Google Cloud Storage are supported.",
     )
     parser.add_argument("backup_id", help="the backup ID")
     parser.add_argument("recovery_dir", help="the path to a directory for recovery.")
-    parser.add_argument(
-        "-V", "--version", action="version", version="%%(prog)s %s" % barman.__version__
-    )
-    parser.add_argument("--help", action="help", help="show this help message and exit")
-    verbosity = parser.add_mutually_exclusive_group()
-    verbosity.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase output verbosity (e.g., -vv is more than -v)",
-    )
-    verbosity.add_argument(
-        "-q",
-        "--quiet",
-        action="count",
-        default=0,
-        help="decrease output verbosity (e.g., -qq is less than -q)",
-    )
-    parser.add_argument(
-        "-t",
-        "--test",
-        help="Test cloud connectivity and exit",
-        action="store_true",
-        default=False,
-    )
     parser.add_argument(
         "--tablespace",
         help="tablespace relocation rule",
@@ -134,24 +103,6 @@ def parse_arguments(args=None):
         action="append",
         default=[],
     )
-    parser.add_argument(
-        "--cloud-provider",
-        help="The cloud provider to use as a storage backend",
-        choices=["aws-s3", "azure-blob-storage"],
-        default="aws-s3",
-    )
-    s3_arguments = parser.add_argument_group(
-        "Extra options for the aws-s3 cloud provider"
-    )
-    s3_arguments.add_argument(
-        "--endpoint-url",
-        help="Override default S3 endpoint URL with the given one",
-    )
-    s3_arguments.add_argument(
-        "-P",
-        "--profile",
-        help="profile name (e.g. INI section in AWS credentials file)",
-    )
     return parser.parse_args(args=args)
 
 
@@ -171,7 +122,7 @@ def tablespace_map(rules):
                 "NAME:LOCATION",
                 rule,
             )
-            raise SystemExit(1)
+            raise CLIErrorExit()
     return tablespaces
 
 
@@ -207,7 +158,7 @@ class CloudBackupDownloader(object):
             logging.error(
                 "Backup %s for server %s does not exists", backup_id, self.server_name
             )
-            raise SystemExit(1)
+            raise OperationErrorExit()
 
         backup_files = self.catalog.get_backup_files(backup_info)
 
@@ -253,7 +204,7 @@ class CloudBackupDownloader(object):
                 logging.error(
                     "Destination %s already exists and it is not empty", target_dir
                 )
-                raise SystemExit(1)
+                raise OperationErrorExit()
             copy_jobs.append([file_info, target_dir])
             for additional_file in file_info.additional_files:
                 copy_jobs.append([additional_file, target_dir])
